import { world, ItemStack } from '@minecraft/server'

world.afterEvents.entityDie.subscribe(e => {
    const { deadEntity, damageSource } = e;

    if (!damageSource.damagingEntity || damageSource.damagingEntity.typeId !== 'minecraft:player') return;
    const player = damageSource.damagingEntity;

    const entityDrops = {
        'minecraft:wither': [
            { item: 'arena:wither_fire', amount: 1, prob: 100, condition: 1 },
            { item: 'rrs:anti_curse_ring', amount: 1, prob: 40, condition: 1 }
        ],
        'minecraft:ender_dragon': [
            { item: 'arena:dragon_scale', amount: 9, prob: 100, condition: 1 }
        ],
        'minecraft:warden': [
            { item: 'arena:warden_heart', amount: 1, prob: 100, condition: 1 },
            { item: 'arena:blindfold', amount: 1, prob: 10, condition: 1 }
        ],
        'minecraft:piglin_brute': [
            { item: 'arena:piglin_bracelet_charm', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:enderman': [
            { item: 'arena:obsidian_eye_talisman', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:villager': [
            { item: 'arena:chained_heart_talisman', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:cow': [
            { item: 'arena:eternal_beef', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:phantom': [
            { item: 'arena:phantom_eye', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:wandering_trader': [
            { item: 'arena:valentine_ring', amount: 1, prob: 25, condition: 1 }
        ],
        'minecraft:cave_spider': [
            { item: 'arena:bezoar', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:wither_skeleton': [
            { item: 'arena:adhesive_bandage', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:stray': [
            { item: 'arena:fast_clock', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:shulker': [
            { item: 'arena:chorus_heart', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:husk': [
            { item: 'arena:forbidden_fruit', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:pillager': [
            { item: 'arena:discount_card', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:witch': [
            { item: 'arena:poisonous_shard', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:slime': [
            { item: 'arena:poisonous_shard', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:bogged': [
            { item: 'arena:poisonous_shard', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:blaze': [
            { item: 'arena:flammable_shard', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:magma_cube': [
            { item: 'arena:flammable_shard', amount: 1, prob: 1, condition: 1 }
        ],
        'minecraft:shulker': [
            { item: 'arena:levitation_shard', amount: 1, prob: 10, condition: 1 }
        ],
        'minecraft:enderman': [
            { item: 'arena:levitation_shard', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:polar_bear': [
            { item: 'arena:cold_shard', amount: 1, prob: 5, condition: 1 }
        ],
        'minecraft:stray': [
            { item: 'arena:cold_shard', amount: 1, prob: 5, condition: 1 },
            { item: 'rrs:freezing_glove', amount: 1, prob: 2, condition: 1 }
        ],
        'minecraft:elder_guardian': [
            { item: 'arena:electric_shard', amount: 3, prob: 100, condition: 1 },
            { item: 'rrs:conduit_necklace', amount: 3, prob: 50, condition: 1 }
        ],
        'minecraft:guardian': [
            { item: 'arena:electric_shard', amount: 1, prob: 20, condition: 1 }
        ],
        'minecraft:drowned': [
            { item: 'arena:electric_shard', amount: 1, prob: 5, condition: 1 }
        ]
    };

    const drops = entityDrops[deadEntity.typeId];
    if (!drops) return;

    drops.forEach(drop => {
        if (!drop.condition) return;

        const randomChance = Math.random() * 100;
        if (randomChance > drop.prob) return;

        try {
            deadEntity.dimension.spawnItem(new ItemStack(drop.item, drop.amount), deadEntity.location);
        } catch {
            player.dimension.spawnItem(new ItemStack(drop.item, drop.amount), player.location);
        }
    });
});
