import { system, world, BlockComponentTypes, BlockTypes } from "@minecraft/server";
import { Vector } from "./Vector3";

const HOLY_GRAIL_EXPERIENCE_COST = 3;
const ROCKET_THRUSTER_EXPERIENCE_COST = 6;
const ROCKET_THRUSTER_COOLDOWN_TIME = 20;
const MAX_VAMPIRE_RING_REGENERATE_HEALTH = 2.5;
const MAX_GOLDEN_SKULL_AMOUNT = 128;
const CHANCE_MULTIPLIER = 1.0;
const MAX_ROCKET_BOOTS_JUMP = 2;
const PIRATE_HAT_LOOT_MULTIPLIER = 3;

const Dimensions = [
    world.getDimension("minecraft:overworld"),
    world.getDimension("minecraft:nether"),
    world.getDimension("minecraft:the_end")
]

const NegativeEffect = [
    "bad_omen",
    "blindness",
    "darkness",
    "fatal_poison",
    "hunger",
    "instant_damage",
    "mining_fatigue",
    "nausea",
    "poison",
    "slowness",
    "weakness",
    "wither"
];

const ScanPattern = [
    { x: -1, y: 0, z: 0 },
    { x: 1, y: 0, z: 0 },
    { x: 0, y: 0, z: 1 },
    { x: 0, y: 0, z: -1 },
    { x: 0, y: 0, z: 0 },
    { x: 1, y: 0, z: -1 },
    { x: 1, y: 0, z: 1 },
    { x: -1, y: 0, z: 1 },
    { x: -1, y: 0, z: -1 }
]

let skull_held = {};
let fall_record = {};

let loot_spawned = {
    "minecraft:overworld": [],
    "minecraft:the_end": [],
    "minecraft:nether": []
}

let killed_with_extra_loot = {
    "minecraft:overworld": [],
    "minecraft:the_end": [],
    "minecraft:nether": []
}

const summoned_list = {
    "minecraft:drowned": "arena:wither_skeleton",
    "minecraft:husk": "arena:wither_skeleton",
    "minecraft:skeleton": "arena:wither_skeleton",
    "minecraft:stray": "arena:wither_skeleton",
    "minecraft:bogged": "arena:wither_skeleton",
    "minecraft:wither_skeleton": "arena:wither_skeleton",
    "minecraft:zombie_pigman": "arena:wither_skeleton",
    "minecraft:zombie_villager": "arena:wither_skeleton",
    "minecraft:zombie": "arena:wither_skeleton"
}

world.afterEvents.entityDie.subscribe(s=>{
    if(s.damageSource.damagingEntity == undefined || s.damageSource.damagingEntity.typeId != "minecraft:player") return;

    let player = s.damageSource.damagingEntity;

    if(player.hasTag("rrs:berserk_ring_ring")){
        let effect = player.getEffect("strength");
        if(effect){
            player.addEffect("strength", 100, { showParticles: false, amplifier: Math.min(5, effect.amplifier + 1) });
        }else{
            player.addEffect("strength", 100, { showParticles: false });
        }
    }

    if(player.hasTag("arena:superstitious_hat_head") && s.deadEntity.typeId != "minecraft:player"){
        killed_with_extra_loot[player.dimension.id].push(s.deadEntity.location);
    }

    if(Math.random() < 0.6*CHANCE_MULTIPLIER && player.hasTag("arena:wither_mask_head") && summoned_list[s.deadEntity.typeId]){
        let entity = s.deadEntity.dimension.spawnEntity(summoned_list[s.deadEntity.typeId], s.deadEntity.location);
        entity.applyImpulse(s.deadEntity.getVelocity());
        entity.getComponent("minecraft:tameable").tame(player);
        entity.setDynamicProperty("timeout_duration", system.currentTick + 2400 - Math.round(Math.random() * 600));

        if(entity.hasComponent("minecraft:variant")){
            entity.getComponent("minecraft:variant").value = s.deadEntity.getComponent("minecraft:variant").value;
        }

        if(entity.hasComponent("minecraft:mark_variant")){
            entity.getComponent("minecraft:mark_variant").value = s.deadEntity.getComponent("minecraft:mark_variant").value;
        }
    }
});

world.afterEvents.entitySpawn.subscribe(s =>{
    if(s.entity.typeId == "minecraft:item" && s.cause == "Spawned" && s.entity.isValid()) loot_spawned[s.entity.dimension.id].push(s.entity);
})

world.afterEvents.entityHurt.subscribe(s=>{
    if(s.damageSource.damagingEntity == undefined || s.damageSource.damagingEntity.typeId != "minecraft:player") return;

    let player = s.damageSource.damagingEntity;
    let health = player.getComponent("minecraft:health");
    
    if(player.hasTag("rrs:vampire_ring_ring")){
        health.setCurrentValue(Math.min(health.currentValue + Math.min(parseInt(s.damage * 0.25), MAX_VAMPIRE_RING_REGENERATE_HEALTH), health.effectiveMax));
    }
});

world.afterEvents.entityHurt.subscribe(s=>{
    let player = s.hurtEntity;
    let fall_regen_ring = 0;
    let cancel_hit = false;

    if(player.hasTag("rrs:fall_regen_ring_ring") && fall_regen_ring < 3 && s.damageSource.cause == "fall"){
        let effect = player.getEffect("regeneration");
        if(effect){
            player.addEffect("regeneration", 200, { showParticles: false, amplifier: effect.amplifier + 1 });
        }else{
            player.addEffect("regeneration", 200, { showParticles: false });
        }
        fall_regen_ring++;
    }

    if(Math.random() < 0.3*CHANCE_MULTIPLIER && player.hasTag("rrs:mirror_amulet") && s.damageSource.damagingEntity != undefined && s.damageSource.damagingEntity.isValid() && s.damageSource.damagingProjectile == undefined){
        let health = player.getComponent("minecraft:health");
        health.setCurrentValue(Math.min(health.currentValue + s.damage, health.effectiveMax));
        s.damageSource.damagingEntity.applyDamage(s.damage, { damagingEntity: player, cause: "magic" });
        cancel_hit = true;
    }

    if(Math.random() < 0.4*CHANCE_MULTIPLIER && player.hasTag("rrs:voodoo_amulet") && (s.damageSource.damagingEntity != undefined || s.damageSource.damagingProjectile != undefined) && !cancel_hit){
        let entities = player.dimension.getEntities({
            location: player.location,
            maxDistance: 6
        });
        
        if(entities.length > 1){ 
            let select = Math.round(Math.random() * entities.length);
            if(entities[select] != undefined && entities[select].id == player.id){
                if(select == 0){
                    select += 1;
                }else{
                    select -= 1;
                }
            }
            if(entities[select] != undefined){
                let health = player.getComponent("minecraft:health");
                health.setCurrentValue(Math.min(health.currentValue + s.damage, health.effectiveMax));
                entities[select].applyDamage(s.damage);
                cancel_hit = true;
            }
        }
    }
    if(player.hasTag("rrs:void_totem_charm") && s.damageSource.cause == "void" && fall_record[player.id] != undefined){
        let record = [ ...fall_record[player.id] ];
        let run_id = system.runInterval(() =>{
            try {
                let data = record.pop();

                player.teleport(data.location, {
                    rotation: data.rotation
                })

            }catch(error) {
                system.clearRun(run_id);
            }
        })
    }

}, { entityTypes: [ "minecraft:player" ]});

world.afterEvents.entityHitEntity.subscribe(s=>{
    if(!s.hitEntity.isValid()) return;
    if(s.damagingEntity.hasTag("rrs:freezing_glove_guantlet")){
        s.hitEntity.addEffect("slowness", 100);
    }

    if(s.damagingEntity.hasTag("arena:pocket_piston_guantlet")){
        let direction = s.damagingEntity.getViewDirection();
        s.hitEntity.applyKnockback(direction.x/Math.abs(direction.y), direction.z/Math.abs(direction.y), 2.8, 0.5);
    }

    if(s.damagingEntity.hasTag("rrs:golden_horn_amulet") && s.hitEntity.matches({ families: [ "illager" ]})){
        s.hitEntity.addEffect("weakness", 100);
    }

    if(s.damagingEntity.hasTag("rrs:holy_dagger_amulet") && s.hitEntity.matches({ families: [ "undead" ]})){
        s.hitEntity.applyDamage(Math.round(Math.random()*3)+1, { damagingEntity: s.damagingEntity, cause: "magic" });
        s.hitEntity.dimension.spawnParticle("rrs:holy_effect", s.hitEntity.location);
    }

    if(s.damagingEntity.hasTag("rrs:golden_skull_head") && skull_held[s.damagingEntity.id] != undefined){
        s.hitEntity.applyDamage(skull_held[s.damagingEntity.id]/8, { damagingEntity: s.damagingEntity, cause: "magic" });
    }
}, { entityTypes: [ "minecraft:player" ]});

world.afterEvents.playerBreakBlock.subscribe(s=>{
    if(s.player.hasTag("rrs:miners_ring_ring") && !miners_ring){
        if(Math.random() < 0.15*CHANCE_MULTIPLIER && s.itemStackBeforeBreak && s.itemStackBeforeBreak.hasTag("minecraft:is_pickaxe") && s.brokenBlockPermutation.hasTag("minecraft:is_pickaxe_item_destructible")){
            
            let effect = s.player.getEffect("haste");
            if(effect){
                s.player.addEffect("haste", 100, { showParticles: false, amplifier: Math.min(2, effect.amplifier + 1) });
            }else{
                s.player.addEffect("haste", 100, { showParticles: false });
            }
        }
    }
});

let player_ground_path = {};
let player_max_height = {};
let player_glide_travel = {};
let looked_enderman = {};
let flee_creeper = {};
let jumping = {};
let sneaking = {};
let can_double_jump = {};
let player_sleep_state = {};
let shoes_duration = {};
let fin_duration = {};
let thruster_cooldown = {};
let last_sleep_time = {};
let boat_y_rotation_offset = {};

system.runInterval(()=>{
    Dimensions.forEach(dimension=>{
        killed_with_extra_loot[dimension.id].forEach(kill_location =>{
            for(let loot of loot_spawned[dimension.id]){
                if(Math.random() < 0.25*CHANCE_MULTIPLIER && Vector.distance(kill_location, loot.location) < 1.25){
                    dimension.spawnItem(loot.getComponent("minecraft:item").itemStack, loot.location);
                }
            }
        });
        killed_with_extra_loot[dimension.id] = [];
        loot_spawned[dimension.id] = [];
    })

    for(let playerData of world.getPlayers()){
        if(!playerData.isValid()) continue;
        let health = playerData.getComponent("minecraft:health");
        let devine_protection_ring = 0;
        let is_click_jump = false;
        let is_click_sneak = false;
        let is_wake_up = false;
        let is_riding = playerData.hasComponent("minecraft:riding");
        if(player_ground_path[playerData.id] == undefined) player_ground_path[playerData.id] = [
            {
                location: playerData.location,
                rotation: playerData.getRotation()
            }
        ];
        if(jumping[playerData.id] == undefined){
            jumping[playerData.id] = playerData.isJumping;
            sneaking[playerData.id] = playerData.isSneaking;
            player_sleep_state[playerData.id] = playerData.isSleeping;
            can_double_jump[playerData.id] = 0;
            shoes_duration[playerData.id] = 0;
            
            fin_duration[playerData.id] = 0;
            thruster_cooldown[playerData.id] = 0;
            last_sleep_time[playerData.id] = 0;
        }
        if(!playerData.isSprinting && shoes_duration[playerData.id] > 0) shoes_duration[playerData.id] -= 1;
        if(!playerData.isSwimming || !playerData.isInWater){
            fin_duration[playerData.id] = 1;
        }
        if(playerData.isSleeping) last_sleep_time[playerData.id] = world.getAbsoluteTime();

        if(!is_riding && boat_y_rotation_offset[playerData] != undefined) delete boat_y_rotation_offset[playerData.id];

        if(playerData.isOnGround){
            can_double_jump[playerData.id] = 0;
            if(fall_record[playerData.id] != undefined) delete fall_record[playerData.id];
            if(player_max_height[playerData.id] != undefined) delete player_max_height[playerData.id];
            
            player_ground_path[playerData.id].push({
                location: playerData.location,
                rotation: playerData.getRotation()
            });
            if(player_ground_path[playerData.id].length > 20) player_ground_path[playerData.id].shift();
        }else{
            if(shoes_duration[playerData.id] > 0) shoes_duration[playerData.id] -= 1;
            if(fall_record[playerData.id] == undefined){
                fall_record[playerData.id] = [ ...player_ground_path[playerData.id] ]
            }
            fall_record[playerData.id].push({
                location: playerData.location,
                rotation: playerData.getRotation()
            });

            if(player_max_height[playerData.id] == undefined){
                player_max_height[playerData.id] = playerData.location.y;
            }else{
                player_max_height[playerData.id] = Math.max(player_max_height[playerData.id], playerData.location.y);
            }
        }
        if(jumping[playerData.id] != playerData.isJumping){
            jumping[playerData.id] = playerData.isJumping;
            if(jumping[playerData.id]) is_click_jump = true;
        }
        if(sneaking[playerData.id] != playerData.isSneaking){
            sneaking[playerData.id] = playerData.isSneaking;
            if(sneaking[playerData.id]) is_click_sneak = true;
        }
        if(player_sleep_state[playerData.id] != playerData.isSleeping){
            player_sleep_state[playerData.id] = playerData.isSleeping;
            if(!player_sleep_state[playerData.id]) is_wake_up = true;
        }
        
        for(let ring of playerData.getTags()){
            if(ring == undefined) continue;

            if(ring == "rrs:falling_resistance_ring_ring" && playerData.isFalling){
                let effect = playerData.getEffect("slow_falling");
                if(!effect || effect.duration <= 1){
                    playerData.addEffect("slow_falling", 1, { showParticles: false });
                }
            }
            
            if(ring == "rrs:devine_protection_ring_ring" && devine_protection_ring < 3 && health.currentValue < 8){
                playerData.addEffect("resistance", 210, { showParticles: false, amplifier: devine_protection_ring });
                devine_protection_ring++;
            }
            
            if(ring == "rrs:conduit_necklace_necklace" && playerData.isInWater){
                playerData.addEffect("conduit_power", 210, { showParticles: false });
            }
            
            if(ring == "rrs:heroic_ring_ring"){
                playerData.addEffect("village_hero", 1, { showParticles: false });
            }
            
            if(ring == "rrs:magnetic_ring_ring"){
                playerData.dimension.getEntities({type: "minecraft:item", location: playerData.location, maxDistance: 8, minDistance: 0.25}).forEach(entity=>{
                    if(entity.isValid()){
                        let direction = Vector.subtract(playerData.location, entity.location).normalize().multiply(0.04);
                        entity.applyImpulse(direction);
                    }
                })
            }
            
            if(ring == "rrs:shadow_sneak_ring_ring" && playerData.isSneaking){
                playerData.addEffect("invisibility", 5, { showParticles: false });
                playerData.addEffect("speed", 60, {showParticles: false });
            }
            
            if(ring == "rrs:anti_curse_ring_ring"){
                NegativeEffect.forEach(effect => {
                    playerData.removeEffect(effect);
                });
            }

            if(ring == "arena:kitty_slippers_feet"){
                let entities = playerData.dimension.getEntities({
                    location: playerData.location,
                    maxDistance: 8,
                    families: [ "creeper" ]
                });
                entities.forEach(entity =>{
                    flee_creeper[entity.id] = system.currentTick + 10;
                })
            }

            if(ring == "vv"){
                if(playerData.isInLava && playerData.isSneaking){
                    playerData.applyKnockback(0, 0, 0, 0.8);
                }
            }

            if(ring == "rrs:deep_sea_ring_ring"){
                if(playerData.isInWater){
                    playerData.addEffect("water_breathing", 60, {showParticles: false });
                }
            }

            if(system.currentTick % 15 == 0 && Math.random() < 0.2*CHANCE_MULTIPLIER && ring == "rrs:holy_necklace_necklace"){
                let entities = playerData.dimension.getEntities({
                    location: playerData.location,
                    maxDistance: 8,
                    families: [ "undead" ]
                });
                
                if(entities.length > 0){
                    let target = entities[Math.floor(Math.random() * entities.length)];
                    target.setOnFire(5, true);
                    target.dimension.spawnParticle("rrs:holy_effect", target.location);
                }
            }

            if(system.currentTick % 3 == 0 && player_max_height[playerData.id] != undefined && playerData.isFalling && ring == "arena:archaic_warden_claws_archaiccharm"){
                let entities = playerData.dimension.getEntities({
                    location: playerData.location,
                    maxDistance: 5
                });
                
                if(entities.length > 1){
                    let distance = Math.abs(playerData.location.y - player_max_height[playerData.id]);
                    distance = Math.round(distance / 8);
                    if(distance > 0) entities.forEach(entity =>{
                        if(entity.id != playerData.id){
                            try {
                                entity.applyDamage(distance, { damagingEntity: playerData, cause: "charging" });
                                if(entity.isOnGround){
                                        entity.applyKnockback(0, 0, 0, 0.65);
                                }
                            }catch(error) {}
                        }
                    })
                }
            }

            if(system.currentTick % 30 == 0 && ring == "rrs:holy_grail_amulet"){
                let total_xp = playerData.getTotalXp();
                let health = playerData.getComponent("minecraft:health");
                if(total_xp > HOLY_GRAIL_EXPERIENCE_COST && health.currentValue < health.effectiveMax){
                    if(playerData.xpEarnedAtCurrentLevel >= HOLY_GRAIL_EXPERIENCE_COST){
                        playerData.addExperience(-HOLY_GRAIL_EXPERIENCE_COST);
                    }else{
                        let sub = HOLY_GRAIL_EXPERIENCE_COST - playerData.xpEarnedAtCurrentLevel;
                        playerData.addLevels(-1);
                        playerData.addExperience(playerData.totalXpNeededForNextLevel - sub);
                    }
                    health.setCurrentValue(Math.min(health.currentValue + 1, health.effectiveMax));
                }
            }

            if(playerData.getComponent('equippable').getEquipment('Chest')?.typeId == 'arena:marble_chestplate' && !playerData.isGliding && !is_riding && playerData.isFalling){
                if(!playerData.isInWater && !playerData.isSneaking && !playerData.isOnGround && !playerData.isFlying && playerData.getVelocity().y <= 0){
                    if(player_glide_travel[playerData.id] == undefined) player_glide_travel[playerData.id] = 0;
                    if(player_glide_travel[playerData.id] > 10){
                        let direction = playerData.getViewDirection();
                        direction = Vector.divide(direction, Math.abs(direction.y));
                        let momentum = Math.min(Math.log10(player_glide_travel[playerData.id]/10 + 1), 1);
    
                        playerData.applyKnockback(direction.x, direction.z, momentum, -0.035/momentum);
                        playerData.dimension.spawnParticle("arena:feather", playerData.location);
                        playerData.addEffect("slow_falling", 1, { showParticles: false });
                    }

                    player_glide_travel[playerData.id] += 1;
                }else{
                    if(player_glide_travel[playerData.id] != undefined) delete player_glide_travel[playerData.id];
                }
            }

            if(ring == "arena:ethereal_sight_mask_head"){
                let entities = playerData.dimension.getEntities({ families: [ "enderman" ], location: playerData.location, maxDistance: 192 });
                entities.forEach(entity =>{
                    let direction = Vector.subtract(entity.location, playerData.location);
                    direction.divide(direction.length());
                    
                    if(Vector.distance(playerData.getViewDirection(), direction) < 1.0) looked_enderman[entity.id] = system.currentTick + 5;
                });
            }

            if(ring == "arena:hermes_boots_feet" && !playerData.isSneaking && !is_riding){
                let velocity = playerData.getVelocity();
                if(velocity.y <= 0.01){
                    let location = playerData.location;
                    location.y -= 0.25;
                    location.x += velocity.x * 2;
                    location.z += velocity.z * 2;
                    let ground = playerData.dimension.getBlock(location);
                    try{
                        if(ground == undefined || ground.isAir) {
                            for(let i = 1; i < 5 - Math.ceil(Math.min(velocity.y, 0)*1.5); i ++){
                                let select = ground.below(i);
                                if(select != undefined && !select.isAir){
                                    ground = select;
                                    break;
                                }
                            }
                        }
                    }catch(error){};
    
                    if(ground != undefined && !ground.isAir){
                        let ancor_location = Vector.add(ground.location, 0.25);
                        ScanPattern.forEach(offset =>{
                            let estim_loc = Vector.add(offset, ancor_location);
                            let block = playerData.dimension.getBlock(estim_loc);
                            if(block != undefined && (block.isLiquid || block.isWaterlogged)){
                                let above = block.above();
                                if(above == undefined || above.isAir){
                                    let entity = playerData.dimension.spawnEntity("arena:dummy_platform", estim_loc);
                                    entity.setDynamicProperty("owner_id", playerData.id);
                                    system.runTimeout(() =>{
                                        if(entity.isValid()) entity.remove();
                                    }, 2);
                                }
                            }
                        })
                    }
                }
            }

            if(ring == "arena:snowshoes_feet" && !playerData.isSneaking && !is_riding){
                let velocity = playerData.getVelocity();
                if(velocity.y <= 0.01){
                    let location = playerData.location;
                    location.y -= 0.25;
                    location.x += velocity.x * 2;
                    location.z += velocity.z * 2;
                    let ground = playerData.dimension.getBlock(location);
                    try{
                        if(ground == undefined || ground.isAir) {
                            for(let i = 1; i < 5 - Math.ceil(Math.min(velocity.y, 0)*1.5); i ++){
                                let select = ground.below(i);
                                if(select != undefined && !select.isAir){
                                    ground = select;
                                    break;
                                }
                            }
                        }
                    }catch(error){};
    
                    if(ground != undefined && !ground.isAir){
                        let ancor_location = Vector.add(ground.location, 0.4);
                        ScanPattern.forEach(offset =>{
                            let estim_loc = Vector.add(offset, ancor_location);
                            let block = playerData.dimension.getBlock(estim_loc);
                            if(block != undefined && block.typeId == "minecraft:powder_snow"){
                                let above = block.above();
                                if(above == undefined || above.isAir){
                                    let entity = playerData.dimension.spawnEntity("arena:dummy_platform", estim_loc);
                                    entity.setDynamicProperty("owner_id", playerData.id);
                                    system.runTimeout(() =>{
                                        if(entity.isValid()) entity.remove();
                                    }, 2);
                                }
                            }
                        })
                    }
                }
            }

            if(ring == "arena:strider_shoes_feet" && !playerData.isSneaking && !is_riding){
                let velocity = playerData.getVelocity();
                if(velocity.y <= 0.01){
                    let location = playerData.location;
                    location.y -= 0.25;
                    location.x += velocity.x * 2;
                    location.z += velocity.z * 2;
                    let ground = playerData.dimension.getBlock(location);
                    try{
                        if(ground == undefined || ground.isAir) {
                            for(let i = 1; i < 5 - Math.ceil(Math.min(velocity.y, 0)*1.5); i ++){
                                let select = ground.below(i);
                                if(select != undefined && !select.isAir){
                                    ground = select;
                                    break;
                                }
                            }
                        }
                    }catch(error){};
    
                    if(ground != undefined && !ground.isAir){
                        let ancor_location = Vector.add(ground.location, 0.45);
                        ScanPattern.forEach(offset =>{
                            let estim_loc = Vector.add(offset, ancor_location);
                            let block = playerData.dimension.getBlock(estim_loc);
                            if(block != undefined && block.typeId == "minecraft:lava"){
                                let above = block.above();
                                if(above == undefined || above.isAir){
                                    let entity = playerData.dimension.spawnEntity("arena:dummy_platform", estim_loc);
                                    entity.setDynamicProperty("owner_id", playerData.id);
                                    system.runTimeout(() =>{
                                        if(entity.isValid()) entity.remove();
                                    }, 2);
                                }
                            }
                        })
                    }
                }
            }

            if(system.currentTick % 5 == 0 && ring == "arena:poseidon_mask_head"){
                let entities = playerData.getEntitiesFromViewDirection({maxDistance: 8, excludeFamilies: [ "monster", "inanimate" ], families: [ "fish", "axolotl", "turtle" ]});
                if(entities.length > 0){
                    let entity;
                    for(let data of entities){
                        let entity_temp = data.entity;
                        if(entity_temp.hasComponent("minecraft:health")){
                            let fish = entity_temp.getComponent("minecraft:health");
                            if(fish.getFamilyTypes().includes("player")){
                                entity = entity_temp;
                                break;
                            }
                        }
                    }

                    if(entity != undefined){
                        let health = entity.getComponent("minecraft:health");
                        let speed = entity.getComponent("minecraft:movement");
                        let text = " " + health.currentValue + "§c - " + health.effectiveMax + "§f, ;" + speed.currentValue.toFixed(2);

                        playerData.onScreenDisplay.setActionBar(text);
                    }
                }
            }

            if(is_click_jump && ring == "rrs:rocket_boots_feet" && !playerData.isFlying && !playerData.isGliding && !is_riding){
                if(can_double_jump[playerData.id] > 0 && can_double_jump[playerData.id] < MAX_ROCKET_BOOTS_JUMP) playerData.applyKnockback(0, 0, 0, 0.5);
                can_double_jump[playerData.id]++;
            }
            
            if(ring == "rrs:running_shoes_feet" && playerData.isSprinting && !playerData.isJumping && !playerData.isGliding && !playerData.isSwimming && !is_riding){
                if(system.currentTick % 4 == 0 && shoes_duration[playerData.id] < 60) shoes_duration[playerData.id] += 1;
                let speed = playerData.getComponent("minecraft:movement");
                let running = shoes_duration[playerData.id] / 40;
                let direction = playerData.getViewDirection();
                direction.y = Math.abs(direction.y);
                direction.x /= direction.y;
                direction.z /= direction.y;
                direction.y = 0;
                
                let face_location = playerData.location;
                face_location.y += 0.6;
                let blocking = playerData.dimension.getBlockFromRay(face_location, direction, { maxDistance: 4, includePassableBlocks: false, includeLiquidBlocks: false});
                if(blocking != undefined){
                    let above_loc = blocking.block.location;
                    above_loc.y += 2;
                    let above = playerData.dimension.getBlockFromRay(above_loc, { x: 0, y: -1, z: 0 }, { maxDistance: 2, includePassableBlocks: false, includeLiquidBlocks: false});
                    if(above != undefined && above.block.location.y == blocking.block.location.y){
                        let blocking_loc = Vector.add(blocking.block.location, blocking.faceLocation);
                        blocking_loc.y = blocking.block.location.y - 0.5;
    
                        let entity = playerData.dimension.spawnEntity("arena:dummy_platform", blocking_loc);
                        entity.setDynamicProperty("owner_id", playerData.id);
                        
                        let run_id = system.runInterval(()=>{
                            if(entity.isValid()) entity.teleport(blocking_loc);
                        })
                        system.runTimeout(() =>{
                            system.clearRun(run_id);
                            if(entity.isValid()) entity.remove();
                        }, 10);
                    }
                }

                if(playerData.isOnGround) playerData.applyKnockback(direction.x, direction.z, speed.currentValue*2 + running, -0.25);
            }
            
            if(ring == "rrs:guardian_fin_taisman" && playerData.isInWater && playerData.isSwimming){
                if(system.currentTick % 4 == 0 && fin_duration[playerData.id] < 60) fin_duration[playerData.id] += 1;
                let speed = playerData.getComponent("minecraft:movement");
                let running = fin_duration[playerData.id] / 40;
                let direction = playerData.getViewDirection();
                let y_dir = Math.sqrt(direction.x**2 + direction.z**2);
                playerData.applyKnockback(direction.x, direction.z, speed.currentValue*2 + running, direction.y * y_dir * (speed.currentValue*2 + running));
            }
            
            if(ring == "rrs:rocket_thruster_body" && is_click_sneak && playerData.isGliding && system.currentTick > thruster_cooldown[playerData.id]){
                let total_xp = playerData.getTotalXp();
                if(total_xp > ROCKET_THRUSTER_EXPERIENCE_COST || playerData.getGameMode() == "creative"){
                    if(playerData.getGameMode() != "creative"){
                        if(playerData.xpEarnedAtCurrentLevel >= ROCKET_THRUSTER_EXPERIENCE_COST){
                            playerData.addExperience(-ROCKET_THRUSTER_EXPERIENCE_COST);
                        }else{
                            let sub = ROCKET_THRUSTER_EXPERIENCE_COST - playerData.xpEarnedAtCurrentLevel;
                            playerData.addLevels(-1);
                            playerData.addExperience(playerData.totalXpNeededForNextLevel - sub);
                        }
                    }
                    thruster_cooldown[playerData.id] = system.currentTick + 26 + ROCKET_THRUSTER_COOLDOWN_TIME;
                    playerData.dimension.playSound("firework.launch", playerData.location);

                    let run_id = system.runInterval(()=>{
                        try{
                            if(!playerData.isGliding || playerData.isOnGround || playerData.isInWater){
                                system.clearRun(run_id);
                                return;
                            }

                            let direction = playerData.getViewDirection();
                            let y_dir = Math.sqrt(direction.x**2 + direction.z**2);
                            playerData.applyKnockback(direction.x, direction.z, 2, direction.y * y_dir * 2);
                            if(system.currentTick % 2 == 0) playerData.dimension.spawnParticle("rrs:thruster_effect", playerData.location);
                        }catch(error){
                            system.clearRun(run_id);
                        }
                    });
    
                    system.runTimeout(()=>{
                        system.clearRun(run_id);
                    }, 25);
                }
            }

            if(ring == "arena:archaic_warden_claws_archaiccharm" && is_wake_up){
                if(world.getAbsoluteTime() - last_sleep_time[playerData.id] > 200){
                    let health = playerData.getComponent("minecraft:health");
                    health.setCurrentValue(health.effectiveMax);
                }
            }
        }
    }
});

world.afterEvents.dataDrivenEntityTrigger.subscribe(s =>{
    if(s.entity.isValid() && s.entity.matches({ families: [ "enderman" ]}) && looked_enderman[s.entity.id] != undefined && s.eventId == "minecraft:become_angry" && system.currentTick < looked_enderman[s.entity.id]){
        s.entity.triggerEvent("minecraft:on_calm");
    }
    if(s.entity.isValid() && s.entity.matches({ families: [ "creeper" ]}) && flee_creeper[s.entity.id] != undefined && s.eventId == "minecraft:start_exploding" && system.currentTick < flee_creeper[s.entity.id]){
        s.entity.triggerEvent("minecraft:stop_exploding");
    }
})

var is_open_loot = {}

world.beforeEvents.playerInteractWithBlock.subscribe(s =>{
    if(s.block != undefined && Math.random() < 0.2*CHANCE_MULTIPLIER && s.player.hasTag("rrs:pirate_hat_head")){
        let container = s.block.getComponent(BlockComponentTypes.Inventory);
        if(container != undefined){
            container = container.container;
            if(container.size == container.emptySlotsCount){
                let id = s.player.id;
                is_open_loot[id] = s.block.location;
                system.runTimeout(() =>{
                    delete is_open_loot[id];
                }, 2);
            }
        }
    }
})

world.afterEvents.playerInteractWithBlock.subscribe(s =>{
    if(is_open_loot[s.player.id] == undefined) return;
    
    if(s.block != undefined){
        let id = s.player.id;
        let dimension = s.player.dimension;

        system.runTimeout(() =>{
            let block = dimension.getBlock(is_open_loot[id]);
            let container = block.getComponent(BlockComponentTypes.Inventory).container;
            if(container.emptySlotsCount < container.size){
                let size = container.size;
                for(let i = 0; i < size; i++){
                    if(Math.random() > 0.4) continue;

                    let item = container.getSlot(i);
                    if(item.hasItem()){
                        item.amount = Math.min(item.maxAmount, Math.round(Math.random() * PIRATE_HAT_LOOT_MULTIPLIER + item.amount));
                    }
                }
            }
        }, 1);
    }
})